@@ -1,9 +1,14 @@
-.\"	Version:
-.ds RE 1.20
+.\"  
+.de @revision
+.ds RE \\$2
+..
+.\"
+.\" $Id: tmac.m,v 1.23 1994/10/31 06:56:57 jh Exp $
+.@revision $Revision: 1.23 $
 .ig
 
-Copyright (C) 1991, 1992, 1993 Free Software Foundation, Inc.
-mgm is written by Jrgen Hgg (jh@efd.lth.se)
+Copyright (C) 1991, 1992, 1993, 1994 Free Software Foundation, Inc.
+mgm is written by Jrgen Hgg <jh@axis.se>
 
 mgm is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -23,7 +28,7 @@ Almost complete. The letter format is not included.
 Maybe as a separate package.
 Should be better as time goes.
 
-Please send bugreports with examples to jh@efd.lth.se.
+Please send bugreports with examples to jh@axis.se.
 
 Naming convention stolen from mgs.
 Local names	module*name
@@ -32,7 +37,7 @@ Env.var		environ:name
 Index		array!index
 ..
 .warn
-.\" ####### init ######
+.\" ######## init #######
 .\"	Contents level [0:7], contents saved if heading level <= Cl
 .nr Cl 2
 .\"	Eject page between LIST OF XXXX if Cp == 0
@@ -229,6 +234,42 @@ Index		array!index
 .\" indent for VERBON
 .nr Verbin 5n
 .\"
+.\" Letter section
+.\" Formal closing (.FC)
+.ds Letfc Yours very truly,
+.\"
+.\" Approval line
+.ds Letapp APPROVED:
+.\" Approval date-string
+.ds Letdate Date
+.\"
+.ds LetCN CONFIDENTIAL\"		Confidential default
+.ds LetSA To Whom It May Concern:\"	Salutation default
+.ds LetAT ATTENTION:\"			Attention string
+.ds LetSJ SUBJECT:\"			Subject string
+.ds LetRN In reference to:\"		Reference string
+.\"
+.\" Copy to (.NS)
+.ds Letns!copy Copy \" space!
+.ds Letns!to " to
+.ds Letns!0 Copy to
+.ds Letns!1 Copy (with att.) to
+.ds Letns!2 Copy (without att.) to
+.ds Letns!3 Att.
+.ds Letns!4 Atts.
+.ds Letns!5 Enc.
+.ds Letns!6 Encs.
+.ds Letns!7 Under separate cover
+.ds Letns!8 Letter to
+.ds Letns!9 Memorandum to
+.ds Letns!10 Copy (with atts.) to
+.ds Letns!11 Copy (without atts.) to
+.ds Letns!12 Abstract Only to
+.ds Letns!13 Complete Memorandum to
+.\"
+.\" Max lines in return address
+.nr Letwam 14
+.\"--------------------------
 .\"	test for mgm macro. This can be used if the text must test
 .\"	what macros is used.
 .nr .mgm 1
@@ -277,6 +318,8 @@ in=\\n[.i] fi=\\n[.u] .d=\\n[.d] nl=\\n[nl] pg=\\n[%]
 .nr hd*last-hpos -1
 .nr par*number 0 1
 .af par*number 01
+.nr par*number2 0 1
+.af par*number2 01
 .nr par*num-count 0 1
 .af par*num-count 01
 .\"	reset numbered paragraphs, arg1 = headerlevel
@@ -291,13 +334,31 @@ in=\\n[.i] fi=\\n[.u] .d=\\n[.d] nl=\\n[nl] pg=\\n[%]
 .ie !((\\n[nl]=\\n[hd*last-pos]):(\\n[nl]=(\\n[hd*last-pos]-.5v))) \{\
 .	if \\n[D]>2 .tm Paragraph 
 .	par@doit \\$*
+.	if \\n[Np] \\n[H1].\\n+[par*number]\ \ \c
 .\}
 .el .if !(\\n[hd*last-hpos]=\\n[.k]) \{\
 .	if \\n[D]>2 .tm Paragraph 
 .	par@doit \\$*
+.	if \\n[Np] \\n[H1].\\n+[par*number]\ \ \c
 .\}
 .nr par@ind-flag 1
 ..
+.\"------------
+.de nP
+.\"	skip P if previous heading
+.ie !((\\n[nl]=\\n[hd*last-pos]):(\\n[nl]=(\\n[hd*last-pos]-.5v))) \{\
+.	if \\n[D]>2 .tm Paragraph 
+.	par@doit \\$*
+\\n[H2].\\n+[par*number2]\ \ \c
+.\}
+.el .if !(\\n[hd*last-hpos]=\\n[.k]) \{\
+.	if \\n[D]>2 .tm Paragraph 
+.	par@doit \\$*
+\\n[H2].\\n+[par*number2]\ \ \c
+.\}
+.nr par@ind-flag 1
+..
+.\"------------
 .de par@doit
 .SP (u;\\n[Ps]*\\n[Lsp])
 .ie  \\n[.$] \{\
@@ -307,10 +368,6 @@ in=\\n[.i] fi=\\n[.u] .d=\\n[.d] nl=\\n[nl] pg=\\n[%]
 .	if \\n[Pt]=1 .ti +\\n[Pi]n
 .	if (\\n[Pt]=2)&\\n[par@ind-flag] .ti +\\n[Pi]n
 .\}
-.if \\n[Np] \{\
-\\n[H1].\\n+[par*number]\ \ \c
-.\" $$$ 'br
-.\}
 ..
 .\" ####### module line #######################################
 .de SP
@@ -862,21 +919,24 @@ in=\\n[.i] fi=\\n[.u] .d=\\n[.d] nl=\\n[nl] pg=\\n[%]
 .\"
 .\" suppress pageheader if pagenumber == 1 and N == [124]
 .if \\n[pg*top-enabled] \{\
-.\" mste fixas!!
+.\"	must be fixed!!
 .\".	pg@disable-top-trap
 .	if \\n[pg*extra-header-size] 'sp \\n[pg*extra-header-size]u
 .	if \\n[pg*top-margin] 'sp \\n[pg*top-margin]u
 .	ev pg*tl-ev
 .	pg@set-env
-.	ie !d TP \{\
-'		sp 3
-.		ie ((\\n[%]=1)&((\\n[N]=1):(\\n[N]=2))) .sp
-.		el .tl \\*[pg*header]
-.		ie o .tl \\*[pg*odd-header]
-.		el .tl \\*[pg*even-header]
-'		sp 2
+.	ie d let@header .let@header
+.	el \{\
+.		ie d TP .TP
+.		el \{\
+'			sp 3
+.			ie ((\\n[%]=1)&((\\n[N]=1):(\\n[N]=2))) .sp
+.			el .tl \\*[pg*header]
+.			ie o .tl \\*[pg*odd-header]
+.			el .tl \\*[pg*even-header]
+'			sp 2
+.		\}
 .	\}
-.	el .TP
 .	ev
 .	\" why no-space??
 .	if d PX \{\
@@ -2427,6 +2487,7 @@ in=\\n[.i] fi=\\n[.u] .d=\\n[.d] nl=\\n[nl] pg=\\n[%]
 .\"
 .nr cov*au 0
 .de TL
+.rm IA IE WA WE LO LT
 .if \\n[.$]>0 .ds cov*title-charge-case \\$1
 .if \\n[.$]>1 .ds cov*title-file-case \\$2
 .pg@disable-top-trap
@@ -2488,9 +2549,10 @@ in=\\n[.i] fi=\\n[.u] .d=\\n[.d] nl=\\n[nl] pg=\\n[%]
 .ie \\n[yr]<50 .ds cov*new-date \\*[MO\\n[mo]] \\n[dy], 20\\n[yr]
 .el .ds cov*new-date \\*[MO\\n[mo]] \\n[dy], 19\\n[yr]
 .als DT cov*new-date
+.ds cov*new-date-set
 .de ND
-.\" don't remember why I did this: .pg@disable-top-trap
 .ds cov*new-date \\$1
+.ds cov*new-date-set \\$1
 ..
 .\"-------------------
 .\" save technical numbers.
@@ -2649,3 +2711,259 @@ in=\\n[.i] fi=\\n[.u] .d=\\n[.d] nl=\\n[nl] pg=\\n[%]
 .	\}
 .\}
 ..
+.\"########################### module let ############################
+.\" Letter macros
+.\"------------------------
+.\" Formal closing
+.de FC
+.ds@print-float 3
+.ne (\\n[cov*au]u+1u)*4v\"	Authors*4v+ 4v
+.in (u;\\n[.l]/2)
+.SP
+.ie \\n[.$] \\$1
+.el \\*[Letfc]
+.in
+..
+.\"------------------------
+.\" Signature line
+.de SG
+.ie d let*type .let*lt-sign \\$@
+.el .let*mt-sign \\$@
+..
+.\"------------------------
+.de let*lt-sign
+.ds@print-float 3
+.ne \\n[let*wa-n]u*4v
+.in (u;\\n[.l]/2)
+.nf
+.nr let*i 0 1
+.while \\n+[let*i]<=\\n[let*wa-n] \{\
+.	SP 3v
+\\*[let*wa-name!\\n[let*i]]
+\\*[let*wa-title!\\n[let*i]]
+.\}
+.fi
+.in
+..
+.\"------------------------
+.\" Memorandum signature
+.de let*mt-sign
+.ds@print-float 3
+.ne \\n[cov*au]u*4v
+.ie \\n[.$]>1 .nr let*k 1
+.el .nr let*k \\n[cov*au]
+.ds let*tmp \\*[cov*au!\\n[let*k]!3]-\\*[cov*au!\\n[let*k]!4]-
+.nr let*i 0 1
+.while \\n+[let*i]<=\\n[cov*au] \{\
+.	if \\n[let*i]>1 .as let*tmp /
+.	as let*tmp \\*[cov*au!\\n[let*k]!2]
+.\}
+.if !''\\$1' .as let*tmp -\\$1
+.in (u;\\n[.l]/2)
+.nf
+.nr let*i 0 1
+.while \\n+[let*i]<=\\n[cov*au] \{\
+.	SP 3v
+.	if \\n[let*i]=\\n[let*k] \{\
+\Z'\h'-(u;\\n[.l]/2)'\\*[let*tmp]'\c
+.	\}
+\\*[cov*au!\\n[let*i]!1]
+.\}
+.fi
+.in
+..
+.\"------------------------
+.\" Approval signature
+.de AV
+.ne 6v
+.nf
+.sp
+.ie \\n[.$]<2 \\*[Letapp]
+.el sp
+.sp 2
+.ie n ______________________________      ______________
+.el \D'l 25m 0'\h'4m'\D'l 12m 0'
+\Z'\\$1'\h'29m'\\*[Letdate]
+.fi
+..
+.\"------------------------
+.\" Letter type
+.\" let@header is called from the header. It is supposed
+.\" to remove the alias itself.
+.de LT
+.rm AF AS AE AT AU CS OK TL MT
+.ds let*type BL
+.nr Pi 5
+.nr Pt 0
+.if !''\\$1' .ds let*type \\$1
+.if !d let@head_\\*[let*type] .@error "LT: unknown letter type \\$1"
+.shift
+.als let@header let@head_\\*[let*type]
+.let@init_\\*[let*type] \\$@
+.if \n[D]>1 .tm Letter type \\*[let*type]
+..
+.\"-----------
+.\" Blocked letter
+.de let@init_BL
+..
+.de let@head_BL
+.rm let@header
+.let@print-head 1
+..
+.\"-----------
+.\" Semiblocked letter
+.de let@init_SB
+.nr Pt 1
+..
+.de let@head_SB
+.rm let@header
+.let@print-head 1
+..
+.\"-----------
+.\" Full-blocked letter
+.de let@init_FB
+..
+.de let@head_FB
+.rm let@header
+.let@print-head
+..
+.\"-----------
+.\" Simplified letter
+.de let@init_SP
+..
+.de let@head_SP
+.rm let@header
+.let@print-head
+..
+.\"--------------------------------------
+.\" Print the letter-head
+.de let@print-head
+.nf
+.sp |11
+.if '1'\\$1' .in (u;\\n[.l]/2)
+.\" ---- WA
+.ie d let@wa-div .let@wa-div
+.el sp 3
+.\" ---- datum
+\\*[cov*new-date-set]
+.sp
+.if '1'\\$1' .if !d let*lo-CN .if !d let*lo-RN .sp 2
+.\" ---- Confidential
+.if d let*lo-CN \{\
+.	ti 0
+.	ie !''\\*[let*lo-CN]' \\*[let*lo-CN]
+.	el \\*[LetCN]
+.	sp
+.\}
+.\" ---- Reference
+.if d let*lo-RN \{\
+\\*[LetRN] \\*[let*lo-RN]
+.	sp
+.\}
+.\" ---- IA
+.sp
+.in 0
+.nr let*i 0 1
+.while \\n+[let*i]<=\\n[let*ia-n] \{\
+\\*[let*ia-name!\\n[let*i]]
+\\*[let*ia-title!\\n[let*i]]
+.\}
+.if d let@ia-div .let@ia-div
+.\" ---- Attention
+.if d let*lo-AT \{\
+.	sp
+\\*[LetAT] \\*[let*lo-AT]
+.\}
+.\" ---- Salutation
+.if !'\\*[let*type]'SP' .if d let*lo-SA \{\
+.	sp
+.	ti 0
+.	ie !''\\*[let*lo-SA]' \\*[let*lo-SA]
+.	el \\*[LetSA]
+.\}
+.\" ---- Subject
+.if d let*lo-SJ \{\
+.	ie '\\*[let*type]'SP' \{\
+.		sp 2
+.		tr aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ
+\\*[let*lo-SJ]
+.		sp
+.		tr aabbccddeeffgghhiijjkkllmmnnooppqqrrssttuuvvwwxxyyzz
+.	\}
+.	el \{\
+.		sp
+.		if '\\*[let*type]'SB' .ti +5m
+\\*[LetSJ] \\*[let*lo-SJ]
+.	\}
+.\}
+..
+.\"-------------------
+.\" .IA [name [title]]
+.nr let*ia-n 0 1
+.de IA
+.if \\n[.$] .ds let*ia-name!\\n+[let*ia-n] \\$1
+.if \\n[.$]>1 .ds let*ia-title!\\n[let*ia-n] \\$2
+.ev let@ev
+'nf
+.di let@ia-div
+.eo
+..
+.de IE
+.di
+.ec
+.ev
+..
+.\"-------------------
+.\" .WA [name [title]]
+.nr let*wa-n 0 1
+.de WA
+.if \\n[.$] .ds let*wa-name!\\n+[let*wa-n] \\$1
+.if \\n[.$]>1 .ds let*wa-title!\\n[let*wa-n] \\$2
+.ev let@ev
+'nf
+.di let@wa-div
+.it \\n[Letwam] let@wa-drain
+.eo
+..
+.\"------
+.de let@wa-drain
+.it
+.di
+.di let@wa-junk
+..
+.\"------
+.de WE
+.it
+.ec
+.di
+.ev
+.if d let@wa-junk .rm let@wa-junk
+..
+.\"-------------------
+.\" Copy to
+.de NS
+.sp
+.nr let*i 0
+.ie !''\\$2' .ds let*str \\$1
+.el \{\
+.	ie !''\\$1' \{\
+.		if d Letns!\\$1 .nr let*i \\$1
+.		ds let*str \\*[Letns!\\n[let*i]]
+.	\}
+.	el .ds let*str \\*[Letns!copy](\\$1)\\*[Letns!to]
+.\}
+.nf
+\\*[let*str]
+..
+.de NE
+.fi
+..
+.\"-------------------
+.\" Letter options
+.de LO
+.rm AF AS AE AT AU CS OK TL MT
+.if ''\\$1' .@error "LO: missing option"
+.if !d Let\\$1 .@error "LO: unknown option (\\$1)"
+.ds let*lo-\\$1 \\$2
+.if \n[D]>1 .tm Letter option \\$1 \\$2
+..
